@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+require "base64"
 require "faraday"
 require "json"
 
@@ -670,6 +671,7 @@ module Google
           Errno::ETIMEDOUT,
           Timeout::Error
         ].freeze
+        private_constant :TRANSIENT_EXCEPTIONS
 
         ##
         # @private
@@ -693,6 +695,7 @@ module Google
         # than the previous old token. See internal issue b/311414224.
         #
         TOKEN_EXPIRY_BUFFER = 210
+        private_constant :TOKEN_EXPIRY_BUFFER
 
         ##
         # @private
@@ -806,7 +809,7 @@ module Google
         #
         def access_token_lifetime data
           json = JSON.parse data rescue nil
-          return 0 unless json&.key? "expires_in"
+          return 0 unless json.respond_to?(:key?) && json.key?("expires_in")
           lifetime = json["expires_in"].to_i - TOKEN_EXPIRY_BUFFER
           lifetime = 0 if lifetime.negative?
           lifetime
@@ -818,9 +821,9 @@ module Google
         #
         def identity_token_lifetime data
           return 0 unless data =~ /^[\w=-]+\.([\w=-]+)\.[\w=-]+$/
-          base64 = Base64.decode64 Regexp.last_match[1]
+          base64 = Base64.urlsafe_decode64 Regexp.last_match[1]
           json = JSON.parse base64 rescue nil
-          return 0 unless json&.key? "exp"
+          return 0 unless json.respond_to?(:key?) && json&.key?("exp")
           lifetime = json["exp"].to_i - Time.now.to_i - TOKEN_EXPIRY_BUFFER
           lifetime = 0 if lifetime.negative?
           lifetime
